package cn.iocoder.yudao.module.system.service.worklog;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.module.system.controller.admin.worklog.vo.loginstance.LogInstancePageReqVO;
import cn.iocoder.yudao.module.system.controller.admin.worklog.vo.loginstance.LogInstanceRespVO;
import cn.iocoder.yudao.module.system.controller.admin.worklog.vo.loginstance.LogInstanceSaveReqVO;
import cn.iocoder.yudao.module.system.convert.worklog.LogInstanceConvert;
import cn.iocoder.yudao.module.system.dal.dataobject.user.AdminUserDO;
import cn.iocoder.yudao.module.system.dal.dataobject.worklog.LogFormDO;
import cn.iocoder.yudao.module.system.dal.dataobject.worklog.LogInstanceDO;
import cn.iocoder.yudao.module.system.dal.dataobject.worklog.LogRuleDO;
import cn.iocoder.yudao.module.system.dal.mysql.worklog.LogInstanceMapper;
import cn.iocoder.yudao.module.system.service.dict.DictDataService;
import cn.iocoder.yudao.module.system.service.user.AdminUserService;
import cn.iocoder.yudao.module.system.service.worklog.dto.LogReadUserRespDTO;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.util.List;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils.getLoginUserId;
import static cn.iocoder.yudao.module.system.enums.ErrorCodeConstants.LOG_INSTANCE_NOT_EXISTS;
import static cn.iocoder.yudao.module.system.enums.ErrorCodeConstants.LOG_RULE_NOT_FIND;

/**
 * 日志实例的拓展 Service 实现类
 *
 * @author 符溶馨
 */
@Service
@Validated
public class LogInstanceServiceImpl implements LogInstanceService {

    @Resource
    private LogInstanceMapper logInstanceMapper;

    @Resource
    private DictDataService dictDataService;

    @Resource
    private LogRuleService logRuleService;

    @Resource
    private LogFormService logFormService;

    @Resource
    private AdminUserService adminUserService;

    @Resource
    private LogReadService logReadService;

    @Override
    public Long createLogInstance(LogInstanceSaveReqVO createReqVO) {

        LogInstanceDO logInstance = BeanUtils.toBean(createReqVO, LogInstanceDO.class);

        //获取日志类型
        LogFormDO logFormDo = logFormService.getForm(createReqVO.getFormId());
        LogRuleDO logRuleDO = logRuleService.getLogRule(logFormDo.getRuleId());
        if (logRuleDO == null) {
            throw exception(LOG_RULE_NOT_FIND);
        }

        //获取部门信息
        AdminUserDO adminUserDO = adminUserService.getUser(getLoginUserId());

        //设置发起人用户编号
        logInstance.setStartUserId(getLoginUserId());
        //设置发起人部门编号
        logInstance.setDeptId(adminUserDO.getDeptId());
        //设置发起人用户名称
        logInstance.setStartUserName(adminUserDO.getNickname());
        //设置发起人用户头像
        logInstance.setAvatar(adminUserDO.getAvatar());
        //设置日志类型
        logInstance.setType(logRuleDO.getType());
        //设置日志名称
        logInstance.setName(adminUserDO.getNickname() + "的" + logFormDo.getName());

        // 插入
        logInstanceMapper.insert(logInstance);

        //创建日志时，查询可以查看发起人日志的用户组  用线程控制
        new Thread(() -> {

            List<LogReadUserRespDTO> respDTOS = logInstanceMapper.selectRaedUser(adminUserDO.getDeptId());
            logReadService.createLogRule(respDTOS, logInstance.getId(), logInstance.getStartUserId());
        }).start();

        // 返回
        return logInstance.getId();
    }

    @Override
    public void updateLogInstance(LogInstanceSaveReqVO updateReqVO) {
        // 校验存在
        validateLogInstanceExists(updateReqVO.getId());
        // 更新
        LogInstanceDO updateObj = BeanUtils.toBean(updateReqVO, LogInstanceDO.class);
        logInstanceMapper.updateById(updateObj);
    }

    @Override
    public void deleteLogInstance(Long id) {
        // 校验存在
        validateLogInstanceExists(id);
        // 删除
        logInstanceMapper.deleteById(id);
    }

    private void validateLogInstanceExists(Long id) {
        if (logInstanceMapper.selectById(id) == null) {
            throw exception(LOG_INSTANCE_NOT_EXISTS);
        }
    }

    @Override
    public LogInstanceDO getLogInstance(Long id) {
        return logInstanceMapper.selectById(id);
    }

    @Override
    public PageResult<LogInstanceDO> getLogInstancePage(LogInstancePageReqVO pageReqVO) {
        return logInstanceMapper.selectPage(pageReqVO, getLoginUserId());
    }

    @Override
    public PageResult<LogInstanceRespVO> getMyProcessInstancePage(Long userId, LogInstancePageReqVO pageReqVO) {

        return LogInstanceConvert.INSTANCE.convertPage(logInstanceMapper.selectMyPage(userId, pageReqVO));
    }
}