package cn.iocoder.yudao.module.system.service.worklog;

import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.module.system.dal.dataobject.worklog.LogUseDO;
import cn.iocoder.yudao.module.system.dal.mysql.worklog.LogUseMapper;
import cn.iocoder.yudao.module.system.service.worklog.dto.LogUseSaveReqDTO;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.system.enums.ErrorCodeConstants.LOG_USE_NOT_EXISTS;

/**
 * 日志模板使用者 Service 实现类
 *
 * @author 符溶馨
 */
@Service
@Validated
public class LogUseServiceImpl implements LogUseService {

    @Resource
    private LogUseMapper logUseMapper;

    @Override
    public Long createLogUse(LogUseSaveReqDTO createReqVO) {
        // 插入
        LogUseDO logUse = BeanUtils.toBean(createReqVO, LogUseDO.class);
        logUseMapper.insert(logUse);
        // 返回
        return logUse.getId();
    }

    @Override
    public void updateLogUse(LogUseSaveReqDTO updateReqVO) {
        // 校验存在
        validateLogUseExists(updateReqVO.getFormId());
        // 更新
        LogUseDO updateObj = BeanUtils.toBean(updateReqVO, LogUseDO.class);
        logUseMapper.updateById(updateObj);
    }

    private void validateLogUseExists(Long formId) {
        if (logUseMapper.selectCount(LogUseDO::getFormId, formId) == 0L) {
            throw exception(LOG_USE_NOT_EXISTS);
        }
    }

    @Override
    public LogUseDO getLogUse(Long id) {
        return logUseMapper.selectById(id);
    }
}